package k2;

import java.awt.Graphics;
import java.awt.Image;
import java.util.Date;
import javax.swing.ImageIcon;
import javax.swing.JLabel;

/**
 * This object both models a 7 segment digit with an annunciator and decimal
 * and also displays it.  Yes, I realize this breaks the traditional
 * model-view-controller paradigm, but you try programming with a wife and 2
 * little boys coming in and talking to you while you're programming.
 * Images blatantly swiped from the K2 manual front page picture are
 * loaded for each segment and then there's a big table lookup for which
 * segments to turn on or off for each character.  Segments are named:
   <pre>
         a
       -----
      |     |
     f|     |b
      |  g  |
       -----
      |     |
     e|     |c
      |     |
       -----      --  dp (decimal point)
         d

         \/  an (annunciator)
   </pre>

*/

public class SevenSegmentLED {

	Image LEDa, LEDb, LEDc, LEDd, LEDe, LEDf, LEDg, LEDdp, LEDan;
	private boolean a, b, c, d, e, f, g, dp, an;
	private boolean flashing; // Is/isn't this annunciator flashing.
	private int flashInterval; // On which second does this annunciator flash.
	private int flashCount; // Counter, annunciator turns on when 0.
	private int flashEvenOdd; // Annunciator 1st first flashes on even/odd sec.
	private int x, y; // Screen coordinate of this SevenSegmentLED.
	private long oldSecond; // Used for blinking annunciators once/second.

	/** Create 7-seg display with everything turned off. */
	SevenSegmentLED() {
		a = b = c = d = e = f = g = dp = an = false;
		// Load images of led segments.
		LEDa = createImage("images/a.png", "a");
		LEDb = createImage("images/b.png", "b");
		LEDc = createImage("images/c.png", "c");
		LEDd = createImage("images/d.png", "d");
		LEDe = createImage("images/e.png", "e");
		LEDf = createImage("images/f.png", "f");
		LEDg = createImage("images/g.png", "g");
		LEDdp = createImage("images/dp.png", "dp");
		LEDan = createImage("images/an.png", "an");
		flashing = false;
		flashEvenOdd = -1;
	}

	/** Set whether or not this annunciator is currently flashing. */
	public void setFlash(boolean f) {
		if (flashing != f) {
			flashing = f;
			flashCount = 0;
			flashInterval = 2;
		}
	}
	/** Get whether or not this annunciator is currently flashing. */
	public boolean getFlash() { return flashing; }
	/** Set the second interval, 0, 1, or 2, that this annunciator can flash. */
	public void setFlashInterval(int i) { flashInterval = i; }
	/** Get the second interval that this annunciator is flashing. */
	public int getFlashInterval() { return flashInterval; }
	/** Set the second interval, even or odd, that this annunciator flashes. */
	public void setFlashEvenOdd(int i) { flashEvenOdd = i; }
	/** Get the second interval, even or odd, that this annunciator flashes. */
	public int getFlashEvenOdd() { return flashEvenOdd; }

	/** Set the digit's x coord. */
	public void setX(int xCoord) { x = xCoord; }
	/** Set the digit's y coord. */
	public void setY(int yCoord) { y = yCoord; }
	/** Set the digit's coords. */
	public void setLocation(int xCoord, int yCoord) {
		x = xCoord;
		y = yCoord;
	}

	/** Make an ImageIcon holding one of the segments. */
	public static ImageIcon createImageIcon(String path,
		String description) {

		java.net.URL imgURL = SevenSegmentLED.class.getResource(path);
		if (imgURL != null)
			return new ImageIcon(imgURL, description);
		else
			System.err.println("Couldn't find file: " + path);
		return null;
	}

	/** Make an Image holding one of the segments. */
	protected static Image createImage(String path, String description) {

		ImageIcon icon = createImageIcon(path, description);
		if (icon == null)
			return null;
		return icon.getImage();

	}

	/** Turn on/off appropriate segments, decimal point, and annunciator. */
	private void decode(char charToDecode, boolean decimalPoint,
		boolean annunciator) {

		dp = decimalPoint;
		an = annunciator;

		switch (charToDecode) {
		case '0':
			a = b = c = d = e = f = true;
			g = false;
			break;
		case '1':
			b = c = true;
			a = d = e = f = g = false;
			break;
		case '2':
			a = b = d = e = g = true;
			c = f = false;
			break;
		case '3':
			a = b = c = d = g = true;
			e = f = false;
			break;
		case '4':
			b = c = f = g = true;
			a = d = e = false;
			break;
		case '5':
			a = c = d = f = g = true;
			b = e = false;
			break;
		case '6':
			a = c = d = e = f = g = true;
			b = false;
			break;
		case '7':
			a = b = c = true;
			d = e = f = g = false;
			break;
		case '8':
			a = b = c = d = e = f = g = true;
			break;
		case '9':
			a = b = c = d = f = g = true;
			e = false;
			break;
		case 'a':
		case 'A':
			a = b = c = e = f = g = true;
			d = false;
			break;
		case 'B':
			c = d = e = f = g = true;
			a = b = false;
			break;
		case 'C': // 'C'
			a = d = e = f = true;
			b = c = g = false;
			break;
		case 'X': // c with a bar on top
			a = d = e = g = true;
			b = c = f = false;
			break;
		case 'Z': // 'C'
			d = e = g = true;
			a = b = c = f = false;
			break;
		case 'D':
			b = c = d = e = g = true;
			a = f = false;
			break;
		case 'E':
			a = d = e = f = g = true;
			b = c = false;
			break;
		case 'F':
			a = e = f = g = true;
			b = c = d = false;
			break;
		case 'G':
			a = c = d = e = f = true;
			b = g = false;
			break;
		case 'H':
			b = c = e = f = g = true;
			a = d = false;
			break;
		case 'K': // 'h'
			c = e = f = g = true;
			a = b = d = false;
			break;
		case 'W':
			c = true;
			a = b = d = e = f = g = false;
			break;
		case 'I':
			e = f = true;
			a = b = c = d = g = false;
			break;
		case 'J':
			b = c = d = true;
			a = e = f = g = false;
			break;
		case '<': // 'l'
			d = e = true;
			a = b = c = f = g = false;
			break;
		case 'L':
			d = e = f = true;
			a = b = c = g = false;
			break;
		case 'M': // 'n'
		case 'N':
			c = e = g = true;
			a = b = d = f = false;
			break;
		case 'O':
			a = b = c = d = e = f = true;
			g = false;
			break;
		case 'P':
			a = b = e = f = g = true;
			c = d = false;
			break;
		case 'Q': // 'o'
			c = d = e = g = true;
			a = b = f = false;
			break;
		case 'q':
			a = b = c = f = g = true;
			d = e = false;
			break;
		case '[': // 'r'
			a = e = g = true;
			b = c = d = f = false;
			break;
		case 'R': // r with a bar on top
			e = g = true;
			a = b = c = d = f = false;
			break;
		case 'S':
			a = c = d = f = g = true;
			b = e = false;
			break;
		case 'T':
			d = e = f = g = true;
			a = b = c = false;
			break;
		case 'U':
			b = c = d = e = f = true;
			a = g = false;
			break;
		case 'V': // 'u'
			c = d = e = true;
			a = b = f = g = false;
			break;
		case 'Y':
			b = c = d = f = g = true;
			a = e = false;
			break;
		case '>':
			g = true;
			a = b = c = d = e = f = false;
			break;
		case '-':
			g = true;
			a = b = c = d = e = f = false;
			break;
		case '_':
			d = true;
			a = b = c = e = f = g = false;
			break;
		case '@':
			a = b = c = d = e = f = g = false;
			break;
		}
	}

	/** Draw the thang. */
	public void display(Graphics grph, JLabel jp, char charToDisplay,
		boolean decimalPoint, boolean annunciator) {

		if (grph == null)
			return;

		// Determine segments that are on and off.
		decode(charToDisplay, decimalPoint, annunciator);

		if (a) grph.drawImage(LEDa, x, y, jp);
		if (b) grph.drawImage(LEDb, x, y, jp);
		if (c) grph.drawImage(LEDc, x, y, jp);
		if (d) grph.drawImage(LEDd, x, y, jp);
		if (e) grph.drawImage(LEDe, x, y, jp);
		if (f) grph.drawImage(LEDf, x, y, jp);
		if (g) grph.drawImage(LEDg, x, y, jp);
		if (dp) grph.drawImage(LEDdp, x, y, jp);
		if (annunciator == false)
			return;
		if (!flashing) {
			if (an) grph.drawImage(LEDan, x, y, jp);
		} else {
			// Flash annunciator as appropriate.
			Date date = new Date();
			long sec = date.getTime() / 1000; // Time in seconds since epoch.
			if (flashEvenOdd != -1) { // Very first start of flash.
				if (sec % 2 == flashEvenOdd) {
					flashEvenOdd = -1;
					grph.drawImage(LEDan, x, y, jp); // Draw annunciator.
					oldSecond = sec;
				}
			} else {
				if (oldSecond != sec) {
					// Blink every flashInterval seconds.  Default is 2.  I
					// added this because FINE RIT has a 3 second pattern:
					// RIT, XIT, blank...but then I found out the DS command
					// doesn't indicate annunciator blinking during FINE RIT.
					// Should software compensate for firmware shortcomings?
					// At the moment I'm not making it compensate.
					flashCount = (flashCount + 1) % flashInterval;
					oldSecond = sec;
				}
				// Subsequent (not the very first) flashes.
				if (flashCount == 0)
					grph.drawImage(LEDan, x, y, jp); // Draw annunciator.
			}
		}
	}

}
