package digital;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;

/**
 * The VUMeter class is a simple model of the geometry and function of a simple
 * vu meter. A VUMeter is always horizontal, has a label to its left, and a
 * measured value to its right. Between the labels is of course the meter
 * measurement area, a bargraph that moves up and down with the measured value.
 *
 * <p>Mike Markowski, ab3ap@arrl.net<br> March 2007
 */
public class VUMeter {

    private Color meterColor, holdColor;
    private Font font = new Font("SansSerif", Font.PLAIN, 16);
    private Rectangle geometry;
    private String name; // Name to be displayed describing meter fn.
    private String units; // E.g., "W", "mV", etc.  Real world units.
    private boolean peakHold, units_W = true;
    private double hold; // For peak and hold.
    private double measurement; // Current measurement.
    private double min, max, nextMax; // Real unit range of meter.
    private int labelX, labelY, msrtX, msrtY; // Label & measurement.
    private int dropCounter, dropCounterMax, holdCounter, holdCounterMax;

    VUMeter() {
        this("Name Me", 0, 0, 10, 10);
    }

    /**
     * The VUMeter constructor has inputs of meter name, to be displayed on
     * screen, and geometry, in pixels, of the rectangular meter. The first
     * arguments, fn1 and fn2, are the K2 command strings needed to indicate a
     * button tap or hold, respectively. The remaining arguments describe where
     * on the image, in pixels, is this particular button.
     */
    VUMeter(String name, int x0, int y0, int xlen, int ylen) {
        this.name = name;
        geometry = new Rectangle(x0, y0, xlen, ylen);
        // Default color is a very light blue.
        meterColor = new Color(0xdb, 0xff, 0xfd);
        // Default hold color is an even lighter light blue.
        holdColor = new Color(0x5e, 0x81, 0x7f);
        measurement = hold = 0.;
        dropCounter = dropCounterMax = 10;
        holdCounter = holdCounterMax = 0;
        peakHold = false;
    }

    /**
     * Indicates if this meter is currently display peak-hold or peak. Peak-hold
     * duration can be specified with setDecayRate().
     */
    public boolean isPeakHold() {
        return peakHold;
    }

    /**
     * Used to specify if meter should display peak-hold or peak.
     */
    public void setPeakHold(char ph) {
        peakHold = (ph == 'N') ? false : true;
    }

    /**
     * Get the name of this meter that is displayed on-screen.
     */
    public String getName() {
        return name;
    }

    /**
     * Set the name of this meter that is displayed on-screen to the left of the
     * meter.
     */
    public void setName(String n) {
        name = n;
    }

    /**
     * Get the x-coordinate, in pixels, of the meter's label.
     */
    public int getLabelX() {
        return labelX;
    }

    /**
     * Set the x-coordinate, in pixels, of the meter's label.
     */
    public void setLabelX(int n) {
        labelX = n;
    }

    /**
     * Get the y-coordinate, in pixels, of the meter's label.
     */
    public int getLabelY() {
        return labelY;
    }

    /**
     * Set the y-coordinate, in pixels, of the meter's label.
     */
    public void setLabelY(int n) {
        labelY = n;
    }

    /**
     * Get the x-coordinate, in pixels, of the meter.
     */
    public double getMeasurementX() {
        return msrtX;
    }

    /**
     * Set the x-coordinate, in pixels, of the meter.
     */
    public void setMeasurementX(int n) {
        msrtX = n;
    }

    /**
     * Get the y-coordinate, in pixels, of the meter.
     */
    public double getMeasurementY() {
        return msrtY;
    }

    /**
     * Set the y-coordinate, in pixels, of the meter.
     */
    public void setMeasurementY(int n) {
        msrtY = n;
    }

    /**
     * Retrieve the meter's current minimum setting, real world units.
     */
    public double getMin() {
        return min;
    }

    /**
     * Set the meter's current minimum setting, real world units.
     */
    public void setMin(double n) {
        min = n;
        measurement = min;
    }

    /**
     * Retrieve the meter's current maximum setting, real world units.
     */
    public double getMax() {
        return max;
    }

    /**
     * Set the meter's current maximum setting, real world units.
     */
    public void setMax(double n) {
        max = (units_W ? n : W_to_dBm(n));
    }

    public void setNextMax(double n) {
        nextMax = (units_W ? n : W_to_dBm(n));
        dropCounter--;
        if (dropCounter <= 0) {
            dropCounter = dropCounterMax;
            if (nextMax != max) {
                max = nextMax;
                hold = 0.;
            }
        }
    }

    public boolean isUnits_W() {
        return units_W;
    }

    public void setUnits_W(boolean units_W) {
        this.units_W = units_W;
        setUnits(units_W ? " W" : "dBm");
    }

    private double W_to_dBm(double watts) {
        double mW = watts * 1e3;
        return 10 * Math.log10(mW);
    }

    /**
     * Retrieve the units string that is tacked onto a measurement.
     */
    public String getUnits() {
        return units;
    }

    /**
     * Set the units string that is tacked onto a measurement.
     */
    public void setUnits(String u) {
        units = new String(u);
    }

    /**
     * Get the x-coordinate, in pixels, of the meter's measurement label.
     */
    public double getUnitsLabelX() {
        return msrtX;
    }

    /**
     * Set the x-coordinate, in pixels, of the meter's measurement label.
     */
    public void setUnitsLabelX(int n) {
        msrtX = n;
    }

    /**
     * Get the y-coordinate, in pixels, of the meter's measurement label.
     */
    public double getUnitsLabelY() {
        return msrtY;
    }

    /**
     * Set the y-coordinate, in pixels, of the meter's measurement label.
     */
    public void setUnitsLabelY(int n) {
        msrtY = n;
    }

    /**
     * Get the Color of the meter and its labels.
     */
    public Color getMeterColor() {
        return meterColor;
    }

    /**
     * Set the Color of the meter and its labels.
     */
    public void setMeterColor(Color c) {
        meterColor = c;
    }

    /**
     * Set the decay rate of peak-hold measurements. The specified "rate" is
     * actually a counter which is decremented at each call to setMeasurement().
     */
    public void setDecayRate(char rate) {
        switch (rate) {
            case 'S': // slow
                holdCounterMax = 5;
                break;
            case 'M': // medium
                holdCounterMax = 4;
                break;
            case 'F': // fast
                holdCounterMax = 3;
                break;
        }
    }

    /**
     * Set the scale drop rate of measurements. The specified "rate" is actually
     * a counter which is decremented at each call to setNextMax(). After the
     * counter reaches zero, the measurement scale will change. The idea is to
     * prevent the scale from changing rapidly due to a brief over- or under-
     * scale reading.
     */
    public void setDropRate(char rate) {
        switch (rate) {
            case 'S': // slow
                dropCounterMax = 5;
                break;
            case 'M': // medium
                dropCounterMax = 3;
                break;
            case 'F': // fast
                dropCounterMax = 1;
                break;
        }
    }

    /**
     * Update the meter with a new measurement, in real world units.
     */
    public void setMeasurement(double m) {
        measurement = m;
        measurement = (units_W ? measurement : W_to_dBm(measurement));
        // Take care of peak-hold decay.
        if (peakHold) {
            holdCounter--;
            if (holdCounter <= 0 || hold < measurement) {
                holdCounter = holdCounterMax;
                hold = Math.max(measurement, hold - 0.1 * max);
                hold = (hold < min) ? min : hold;
                hold = (hold > max) ? max : hold;
            }
        }
    }

    /**
     * Return the buttons image location and size, in pixels.
     */
    public Rectangle getGeometry() {
        return geometry;
    }

    /**
     * Specify the buttons image location and size, in pixels.
     */
    public void setGeometry(Rectangle r) {
        geometry = r;
    }

    /**
     * Specify the buttons image location and size, in pixels.
     */
    public void setGeometry(int x0, int y0, int xlen, int ylen) {
        geometry = new Rectangle(x0, y0, xlen, ylen);
    }

    /**
     * Display the meter with it's labels, measurement, and peak-hold reading,
     * if any.
     */
    public void display(Graphics g) {
        g.setFont(font);
        g.setColor(meterColor);

        // Display label.
        g.drawString(name, labelX, labelY);

        // Display meter.
        if (measurement < min) {
            measurement = min;
        } else if (measurement > max) {
            measurement = max;
        }
        int width = (int) ((measurement - min) / (max - min)
                * geometry.getWidth());
        g.fillRect((int) geometry.getX(), (int) geometry.getY(),
                width, (int) (geometry.getHeight()));

        // Display measurement.
        String unitsLabel;
        if (units.isEmpty()) // Hack!  Shouldn't check this directly.
        {
            unitsLabel = String.format("%4.1f", measurement);
        } else if (measurement < 10.) {
            unitsLabel = String.format("%4.2f", measurement);
        } else if (measurement < 100.) {
            unitsLabel = String.format("%4.1f", measurement);
        } else {
            unitsLabel = String.format("%3d", (int) measurement);
        }
        unitsLabel += " " + units;
        g.drawString(unitsLabel, msrtX, msrtY);

        // Display remembered peak, if any.
        if (peakHold) {
            g.setColor(holdColor);
            int spot = (int) (geometry.getX()
                    + (hold - min) / (max - min) * geometry.getWidth());
            if (spot > geometry.getX()) {
                g.fillRect(spot - 3, (int) geometry.getY(),
                        3, (int) geometry.getHeight());
            }
        }
    }
}
